@@ -2,7 +2,8 @@
 
 ;; Copyright (C) 2017  fmdkdd
 ;; URL: https://github.com/emacs-lsp/lsp-ui
-;; Keywords: lsp, ui
+;; Keywords: languagues, tools
+;; Version: 6.2
 
 ;; This program is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -26,11 +27,13 @@
 
 ;;; Code:
 
-(require 'lsp-mode)
 (require 'flycheck)
 (require 'pcase)
 (require 'dash)
 
+(require 'lsp-protocol)
+(require 'lsp-mode)
+
 (defgroup lsp-ui-flycheck nil
   "The LSP extension to display syntax checking."
   :group 'tools
@@ -39,18 +42,6 @@
   :link '(custom-manual "(lsp-ui-flycheck) Top")
   :link '(info-link "(lsp-ui-flycheck) Customizing"))
 
-(defcustom lsp-ui-flycheck-enable nil
-  "Whether or not to enable ‘lsp-ui-flycheck’."
-  :type 'boolean
-  :group 'lsp-ui)
-
-(defcustom lsp-ui-flycheck-live-reporting t
-  "If non-nil, diagnostics in buffer will be reported as soon as possible.
-Typically, on every keystroke.
-If nil, diagnostics will be reported according to `flycheck-check-syntax-automatically'."
-  :type 'boolean
-  :group 'lsp-ui-flycheck)
-
 (defcustom lsp-ui-flycheck-list-position 'bottom
   "Position where `lsp-ui-flycheck-list' will show diagnostics for the whole workspace."
   :type '(choice (const :tag "Bottom" bottom)
@@ -65,6 +56,8 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
     (forward-line -1)))
 
 (defun lsp-ui-flycheck-list--update (window workspace)
+  "Update flycheck buffer in WINDOW belonging to WORKSPACE.
+Use `lsp-diagnostics' to receive diagnostics from your LSP server."
   (let ((buffer-read-only nil)
         (lsp--cur-workspace workspace))
     (erase-buffer)
@@ -79,15 +72,17 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
                                       'face 'dired-directory)
                           (propertize "\n" 'face '(:height 0.2)))))
                (dolist (diag diagnostic)
-                 (let* ((message (or (lsp-diagnostic-message diag) "???"))
-                        (severity (or (lsp-diagnostic-severity diag) 1))
-                        (line (or (lsp-diagnostic-line diag) 1))
-                        (face (cond ((= severity 1) 'error)
-                                    ((= severity 2) 'warning)
-                                    (t 'success)))
-                        (text (concat (propertize (number-to-string line) 'face face)
-                                      ": "
-                                      (car (split-string message "\n")))))
+                 (-let* (((&Diagnostic :message :severity? :source?
+                                       :range (&Range :start (&Position :line start-line))) diag)
+                         (formatted-message (or (if source? (format "%s: %s" source? message) message) "???"))
+                         (severity (or severity? 1))
+                         (line (1+ start-line))
+                         (face (cond ((= severity 1) 'error)
+                                     ((= severity 2) 'warning)
+                                     (t 'success)))
+                         (text (concat (propertize (number-to-string line) 'face face)
+                                       ": "
+                                       (car (split-string formatted-message "\n")))))
                    (add-text-properties 0 (length text) `(diag ,diag file ,file window ,window) text)
                    (insert (concat text "\n")))))
              (lsp-diagnostics)))
@@ -105,7 +100,7 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
         (window (selected-window)))
     (with-current-buffer buffer
       (lsp-ui-flycheck-list--update window workspace))
-    (add-hook 'lsp-after-diagnostics-hook 'lsp-ui-flycheck-list--refresh nil t)
+    (add-hook 'lsp-diagnostics-updated-hook 'lsp-ui-flycheck-list--refresh nil t)
     (setq lsp-ui-flycheck-list--buffer buffer)
     (let ((win (display-buffer-in-side-window
                 buffer `((side . ,lsp-ui-flycheck-list-position) (slot . 5) (window-width . 0.20)))))
@@ -125,10 +120,10 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
 
 (defun lsp-ui-flycheck-list--open ()
   (-when-let* ((diag (get-text-property (point) 'diag))
+               ((&Diagnostic :range (&Range :start (&Position :line start-line
+                                                              :character start-column))) diag)
                (file (get-text-property (point) 'file))
                (window (get-text-property (point) 'window))
-               (line (lsp-diagnostic-line diag))
-               (column (lsp-diagnostic-column diag))
                (marker (with-current-buffer
                            (or (get-file-buffer file)
                                (find-file-noselect file))
@@ -136,8 +131,8 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
                            (widen)
                            (save-excursion
                              (goto-char 1)
-                             (forward-line line)
-                             (forward-char column)
+                             (forward-line start-line)
+                             (forward-char start-column)
                              (point-marker))))))
     (set-window-buffer window (marker-buffer marker) t)
     (with-selected-window window
@@ -173,69 +168,6 @@ If nil, diagnostics will be reported according to `flycheck-check-syntax-automat
   (setq mode-line-format nil)
   (add-hook 'post-command-hook 'lsp-ui-flycheck-list--post-command nil t))
 
-(defun lsp-ui-flycheck--start (checker callback)
-  "Start an LSP syntax check with CHECKER.
-
-CALLBACK is the status callback passed by Flycheck."
-  ;; Turn all errors from lsp-diagnostics for the current buffer into
-  ;; flycheck-error objects and pass them immediately to the callback
-  (let ((errors))
-    (dolist (diag (or
-                   (gethash (lsp--fix-path-casing buffer-file-name)
-                            (lsp-diagnostics))
-                   (gethash (lsp--fix-path-casing (file-truename buffer-file-name))
-                            (lsp-diagnostics))))
-      (push (flycheck-error-new
-             :buffer (current-buffer)
-             :checker checker
-             :filename buffer-file-name
-             :line (1+ (lsp-diagnostic-line diag))
-             :column (1+ (lsp-diagnostic-column diag))
-             :message (lsp-diagnostic-message diag)
-             :level (pcase (lsp-diagnostic-severity diag)
-                      (1 'error)
-                      (2 'warning)
-                      (_ 'info))
-             :id (lsp-diagnostic-code diag))
-            errors))
-    (funcall callback 'finished errors)))
-
-(flycheck-define-generic-checker 'lsp-ui
-  "A syntax checker using the Language Server Protocol (LSP)
-provided by lsp-mode.
-
-See https://github.com/emacs-lsp/lsp-mode."
-  :start #'lsp-ui-flycheck--start
-  :modes '(python-mode) ; Need a default mode
-  :predicate (lambda () lsp-mode)
-  :error-explainer (lambda (e) (flycheck-error-message e)))
-
-(defun lsp-ui-flycheck-add-mode (mode)
-  "Add MODE as a valid major mode for the lsp checker."
-  (unless (flycheck-checker-supports-major-mode-p 'lsp-ui mode)
-    (flycheck-add-mode 'lsp-ui mode)))
-
-(defun lsp-ui-flycheck--report nil
-  "This callback is invoked when new diagnostics are received from the language server.  Invoke flycheck-buffer to update the display of errors if flycheck-mode is on and we are live reporting or we are in save-mode and the buffer is not modified."
-  (and flycheck-mode
-       (or lsp-ui-flycheck-live-reporting
-	   (and lsp-ui-flycheck--save-mode (not (buffer-modified-p))))
-       (flycheck-buffer)))
-
-;; FIXME: Provide a way to disable lsp-ui-flycheck
-(defun lsp-ui-flycheck-enable (_)
-  "Enable flycheck integration for the current buffer."
-  (setq-local lsp-ui-flycheck--save-mode
-	       (or (memq 'save flycheck-check-syntax-automatically)
-		   lsp-ui-flycheck--save-mode))
-  (setq-local flycheck-check-syntax-automatically nil)
-  (setq-local flycheck-checker 'lsp-ui)
-  (lsp-ui-flycheck-add-mode major-mode)
-  (add-to-list 'flycheck-checkers 'lsp-ui)
-  (add-hook 'lsp-after-diagnostics-hook 'lsp-ui-flycheck--report nil t))
-
-;; lsp-ui.el loads lsp-ui-flycheck.el, so we can’t ‘require’ lsp-ui.
-;; FIXME: Remove this cyclic dependency.
 (declare-function lsp-ui--workspace-path "lsp-ui" (path))
 
 (provide 'lsp-ui-flycheck)
