@@ -4,7 +4,8 @@
 
 ;; Author: Sebastien Chapuis <sebastien@chapu.is>
 ;; URL: https://github.com/emacs-lsp/lsp-ui
-;; Keywords: lsp, ui
+;; Keywords: languages, tools
+;; Version: 6.2
 
 ;;; License
 ;;
@@ -29,6 +30,7 @@
 
 ;;; Code:
 
+(require 'lsp-protocol)
 (require 'lsp-mode)
 (require 'flycheck nil 'noerror)
 (require 'dash)
@@ -118,7 +120,7 @@ This can be used to insert, for example, an unicode character: 💡")
 
 (defvar-local lsp-ui-sideline--tag nil
   "Tag marking where the last operation was based.
-It is used to know when the cursor has changed of line or point.")
+It is used to know when the cursor has changed its line or point.")
 
 (defvar-local lsp-ui-sideline--last-width nil
   "Value of window's width on the last operation.
@@ -126,6 +128,9 @@ It is used to know when the window has changed of width.")
 
 (defvar-local lsp-ui-sideline--timer nil)
 
+(defvar-local lsp-ui-sideline--code-actions nil
+  "Holds the latest code actions.")
+
 (defface lsp-ui-sideline-symbol
   '((t :foreground "grey"
        :box (:line-width -1 :color "grey")
@@ -201,7 +206,7 @@ if OFFSET is non-nil, it starts search OFFSET lines from user point line."
         (lsp-ui-sideline--find-line str-len bol eol nil offset)
       (and pos (or (> pos eol) (< pos bol))
            (push pos lsp-ui-sideline--occupied-lines)
-           (list pos index)))))
+           (list pos (1- index))))))
 
 (defun lsp-ui-sideline--delete-ov ()
   "Delete overlays."
@@ -216,20 +221,20 @@ We prioritize string with a language (which is probably a type or a
 function signature)."
   (when contents
     (cond
-     ((stringp contents) contents)
-     ((sequencep contents) ;; MarkedString[]
-      (seq-find (lambda (it) (and (hash-table-p it)
-                                  (lsp-get-renderer (gethash "language" it))))
+     ((lsp-marked-string? contents) contents)
+     ((vectorp contents)
+      (seq-find (lambda (it) (and (lsp-marked-string? it)
+                                  (lsp-get-renderer (lsp:marked-string-language it))))
                 contents))
-     ((gethash "kind" contents) contents) ;; MarkupContent
-     ((gethash "language" contents) contents))))
+     ((lsp-markup-content? contents) contents))))
+
 
 (defun lsp-ui-sideline--format-info (marked-string)
   "Format MARKED-STRING.
 If the string has a language, we fontify it with the function provided
 by `lsp-mode'.
 MARKED-STRING is the string returned by `lsp-ui-sideline--extract-info'."
-  (when (and marked-string (hash-table-p marked-string))
+  (when (and marked-string (or (lsp-marked-string? marked-string) (lsp-markup-content? marked-string)))
     (setq marked-string (lsp--render-element marked-string))
     (add-face-text-property 0 (length marked-string) 'lsp-ui-sideline-symbol-info nil marked-string)
     (add-face-text-property 0 (length marked-string) 'default t marked-string)
@@ -253,9 +258,11 @@ CURRENT is non-nil when the point is on the symbol."
     (add-face-text-property 0 len 'lsp-ui-sideline-global nil str)
     (concat
      (propertize " " 'display `(space :align-to (- right-fringe ,(lsp-ui-sideline--align len margin))))
-     str)))
+     (propertize str 'display '(height 1)))))
 
 (defun lsp-ui-sideline--check-duplicate (symbol info)
+  "Check if there's already a SYMBOL containing INFO, unless `lsp-ui-sideline-ignore-duplicate'
+is set to t."
   (not (when lsp-ui-sideline-ignore-duplicate
          (--any (and (string= (overlay-get it 'symbol) symbol)
                      (string= (overlay-get it 'info) info))
@@ -286,7 +293,7 @@ CURRENT is non-nil when the point is on the symbol."
 (defun lsp-ui-sideline--push-info (symbol tag bounds info bol eol)
   (when (and (= tag (lsp-ui-sideline--calculate-tag))
              (not (lsp-ui-sideline--stop-p)))
-    (let* ((info (concat (-some->> (gethash "contents" info)
+    (let* ((info (concat (-some->> (lsp:hover-contents info)
                            lsp-ui-sideline--extract-info
                            lsp-ui-sideline--format-info
                            (replace-regexp-in-string "\r" ""))))
@@ -339,35 +346,35 @@ CURRENT is non-nil when the point is on the symbol."
              lines))
 
 (defun lsp-ui-sideline--diagnostics (bol eol)
-  "Show diagnostics on the current line."
+  "Show diagnostics belonging to the current line.
+Loop over flycheck errors with `flycheck-overlay-errors-in'.
+Find appropriate position for sideline overlays with `lsp-ui-sideline--find-line'.
+Push sideline overlays on `lsp-ui-sideline--ovs'."
   (when (bound-and-true-p flycheck-mode)
-      (dolist (e (flycheck-overlay-errors-in bol (1+ eol)))
-        (let* ((lines (--> (flycheck-error-format-message-and-id e)
-                           (split-string it "\n")
-                           (lsp-ui-sideline--split-long-lines it)))
-               (display-lines (butlast lines (- (length lines) lsp-ui-sideline-diagnostic-max-lines)))
-               (offset 1))
-          (dolist (line display-lines)
-            (let* ((message (string-trim (replace-regexp-in-string "[\t ]+" " " line)))
-                   (len (length message))
-                   (level (flycheck-error-level e))
-                   (face (if (eq level 'info) 'success level))
-                   (margin (lsp-ui-sideline--margin-width))
-                   (message (progn (add-face-text-property 0 len 'lsp-ui-sideline-global nil message)
-                                   (add-face-text-property 0 len face nil message)
-                                   message))
-                   (string (concat (propertize " " 'display `(space :align-to (- right-fringe ,(lsp-ui-sideline--align len margin))))
-                                   message))
-                   (pos-ov (lsp-ui-sideline--find-line len bol eol nil offset))
-                   (ov (and pos-ov (make-overlay (car pos-ov) (car pos-ov))))
-                   )
-              (when pos-ov
-                (setq offset (car (cdr pos-ov)))
-                (overlay-put ov 'after-string string)
-                (overlay-put ov 'kind 'diagnotics)
-                (push ov lsp-ui-sideline--ovs))))))))
-
-(defvar-local lsp-ui-sideline--code-actions nil)
+    (dolist (e (flycheck-overlay-errors-in bol (1+ eol)))
+      (let* ((lines (--> (flycheck-error-format-message-and-id e)
+                         (split-string it "\n")
+                         (lsp-ui-sideline--split-long-lines it)))
+             (display-lines (butlast lines (- (length lines) lsp-ui-sideline-diagnostic-max-lines)))
+             (offset 1))
+        (dolist (line display-lines)
+          (let* ((message (string-trim (replace-regexp-in-string "[\t ]+" " " line)))
+                 (len (length message))
+                 (level (flycheck-error-level e))
+                 (face (if (eq level 'info) 'success level))
+                 (margin (lsp-ui-sideline--margin-width))
+                 (message (progn (add-face-text-property 0 len 'lsp-ui-sideline-global nil message)
+                                 (add-face-text-property 0 len face nil message)
+                                 message))
+                 (string (concat (propertize " " 'display `(space :align-to (- right-fringe ,(lsp-ui-sideline--align len margin))))
+                                 (propertize message 'display '(height 1))))
+                 (pos-ov (lsp-ui-sideline--find-line len bol eol nil offset))
+                 (ov (and pos-ov (make-overlay (car pos-ov) (car pos-ov)))))
+            (when pos-ov
+              (setq offset (car (cdr pos-ov)))
+              (overlay-put ov 'after-string string)
+              (overlay-put ov 'kind 'diagnotics)
+              (push ov lsp-ui-sideline--ovs))))))))
 
 (defun lsp-ui-sideline-apply-code-actions nil
   "Choose and apply code action(s) on the current line."
@@ -379,10 +386,10 @@ CURRENT is non-nil when the point is on the symbol."
 (defun lsp-ui-sideline--code-actions (actions bol eol)
   "Show code ACTIONS."
   (when lsp-ui-sideline-actions-kind-regex
-    (setq actions (seq-filter (-lambda ((&hash "kind"))
-                                (or (not kind)
-                                    (s-match lsp-ui-sideline-actions-kind-regex kind)))
-                             actions)))
+    (setq actions (seq-filter (-lambda ((&CodeAction :kind?))
+                                (or (not kind?)
+                                    (s-match lsp-ui-sideline-actions-kind-regex kind?)))
+                              actions)))
   (setq lsp-ui-sideline--code-actions actions)
   (dolist (ov lsp-ui-sideline--ovs)
     (when (eq (overlay-get ov 'kind) 'actions)
@@ -390,7 +397,7 @@ CURRENT is non-nil when the point is on the symbol."
             (delq (overlay-get ov 'position) lsp-ui-sideline--occupied-lines))
       (delete-overlay ov)))
   (seq-doseq (action actions)
-    (-let* ((title (->> (gethash "title" action)
+    (-let* ((title (->> (lsp:code-action-title action)
                         (replace-regexp-in-string "[\n\t ]+" " ")
                         (concat lsp-ui-sideline-code-actions-prefix)))
             (margin (lsp-ui-sideline--margin-width))
@@ -405,7 +412,7 @@ CURRENT is non-nil when the point is on the symbol."
                           (add-text-properties 0 len `(keymap ,keymap mouse-face highlight) title)
                           title))
             (string (concat (propertize " " 'display `(space :align-to (- right-fringe ,(lsp-ui-sideline--align len margin))))
-                            title))
+                            (propertize title 'display '(height 1))))
             (pos-ov (lsp-ui-sideline--find-line (1+ (length title)) bol eol t))
             (ov (and pos-ov (make-overlay (car pos-ov) (car pos-ov)))))
       (when pos-ov
@@ -439,15 +446,16 @@ from the language server."
           (lsp-ui-sideline--diagnostics bol eol))
         (when (and lsp-ui-sideline-show-code-actions (or (lsp--capability "codeActionProvider")
                                                          (lsp--registered-capability "textDocument/codeAction")))
-          (lsp--send-request-async (lsp--make-request
-                                    "textDocument/codeAction"
-                                    (if (equal lsp-ui-sideline-update-mode 'line)
-                                        (list :textDocument doc-id
-                                              :range (lsp--region-to-range bol eol)
-                                              :context (list :diagnostics (lsp-cur-line-diagnostics)))
-                                      (lsp--text-document-code-action-params)))
-                                   (lambda (actions) (lsp-ui-sideline--code-actions actions bol eol))
-                                   'alive))
+          (lsp-request-async
+           "textDocument/codeAction"
+           (if (equal lsp-ui-sideline-update-mode 'line)
+               (list :textDocument doc-id
+                     :range (lsp--region-to-range bol eol)
+                     :context (list :diagnostics (lsp-cur-line-diagnostics)))
+             (lsp--text-document-code-action-params))
+           (lambda (actions) (lsp-ui-sideline--code-actions actions bol eol))
+           :mode 'alive
+           :cancel-token :lsp-ui-code-actions))
         ;; Go through all symbols and request hover information.  Note that the symbols are
         ;; traversed backwards as `forward-symbol' with a positive argument will jump just past the
         ;; current symbol.  By going from the end of the line towards the front, point will be placed
@@ -466,23 +474,25 @@ from the language server."
                      (outside-comment (eq (nth 4 parsing-state) nil)))
                 ;; Skip strings and comments
                 (when (and symbol (not in-string) outside-comment)
-                  (push (list symbol tag bounds (lsp--position (1- line-widen) (- (point) bol))) symbols))))
+                  (push (list symbol tag bounds (list :line (1- line-widen) :character (- (point) bol))) symbols))))
             (seq-do #'lsp--cancel-request lsp-ui-sideline--requests)
             (dolist (ov lsp-ui-sideline--ovs)
               (when (eq (overlay-get ov 'kind) 'info)
                 (setq lsp-ui-sideline--occupied-lines
-                  (delq (overlay-get ov 'position) lsp-ui-sideline--occupied-lines))
+                      (delq (overlay-get ov 'position) lsp-ui-sideline--occupied-lines))
                 (delete-overlay ov)))
             (setq lsp-ui-sideline--requests
-              (mapcar (lambda (it)
-                        (-let (((symbol tag bounds position) it))
-                          (plist-get (lsp-request-async
-                                       "textDocument/hover"
-                                       (list :textDocument doc-id :position position)
-                                       (lambda (info)
-                                         (when info (lsp-ui-sideline--push-info symbol tag bounds info bol eol))))
-                            :id)))
-                symbols))))))))
+                  (mapcar (lambda (it)
+                            (-let (((symbol tag bounds position) it))
+                              (plist-get (lsp-request-async
+                                          "textDocument/hover"
+                                          (lsp-make-hover-params :text-document doc-id
+                                                                 :position position)
+                                          (lambda (info)
+                                            (when info (lsp-ui-sideline--push-info symbol tag bounds info bol eol)))
+                                          :mode 'alive)
+                                         :id)))
+                          symbols))))))))
 
 (defun lsp-ui-sideline--stop-p ()
   "Return non-nil if the sideline should not be display."
@@ -533,6 +543,7 @@ This does not toggle display of flycheck diagnostics or code actions."
   (lsp-ui-sideline))
 
 (defun lsp-ui-sideline--erase (&rest _)
+  "Remove all sideline overlays and delete last tag."
   (when (bound-and-true-p lsp-ui-sideline-mode)
     (ignore-errors
       (lsp-ui-sideline--delete-ov)
@@ -549,7 +560,7 @@ This does not toggle display of flycheck diagnostics or code actions."
    (lsp-ui-sideline-mode
     (add-hook 'post-command-hook 'lsp-ui-sideline nil t)
     (advice-add 'company-pseudo-tooltip-frontend :before 'lsp-ui-sideline--hide-before-company)
-    (add-hook 'lsp-after-diagnostics-hook 'lsp-ui-sideline--diagnostics-changed nil t)
+    (add-hook 'flycheck-after-syntax-check-hook 'lsp-ui-sideline--diagnostics-changed nil t)
     (dolist (cmd lsp-ui-sideline-cmd-erase)
       (advice-add cmd :before 'lsp-ui-sideline--erase))
     (when lsp-ui-sideline-show-diagnostics
@@ -558,7 +569,7 @@ This does not toggle display of flycheck diagnostics or code actions."
     (setq lsp-ui-sideline--tag nil)
     (advice-remove 'company-pseudo-tooltip-frontend 'lsp-ui-sideline--hide-before-company)
     (lsp-ui-sideline--delete-ov)
-    (remove-hook 'lsp-after-diagnostics-hook 'lsp-ui-sideline--diagnostics-changed t)
+    (remove-hook 'flycheck-after-syntax-check-hook  'lsp-ui-sideline--diagnostics-changed t)
     (remove-hook 'post-command-hook 'lsp-ui-sideline t)
     (dolist (cmd lsp-ui-sideline-cmd-erase)
       (advice-remove cmd 'lsp-ui-sideline--erase))
